@@ -54,6 +54,7 @@ struct _GnomeDItemEditPrivate {
 	GtkWidget *child2;
 
         GtkWidget *name_entry;
+        GtkWidget *generic_name_entry;
         GtkWidget *comment_entry;
         GtkWidget *exec_label;
         GtkWidget *exec_entry;
@@ -62,7 +63,7 @@ struct _GnomeDItemEditPrivate {
         GtkWidget *doc_entry;
 
         GtkWidget *type_label;
-        GtkWidget *type_combo;
+        GtkWidget *type_option;
 
         GtkWidget *terminal_button;  
 
@@ -71,7 +72,9 @@ struct _GnomeDItemEditPrivate {
         GtkWidget *translations;
         GtkWidget *transl_lang_entry;
         GtkWidget *transl_name_entry;
+        GtkWidget *transl_generic_name_entry;
         GtkWidget *transl_comment_entry;
+        GtkWidget *transl_icon_entry;
 };
 
 static void gnome_ditem_edit_class_init   (GnomeDItemEditClass *klass);
@@ -158,6 +161,7 @@ gnome_ditem_edit_class_init (GnomeDItemEditClass *klass)
 
 	/* Always get translations in UTF-8, needed if library is used
 	 * outside of gnome-core */
+        bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 }
 
@@ -167,42 +171,87 @@ enum {
 	ALL_EXCEPT_DIRECTORY
 };
 
+#define TYPE_STRING "GnomeDitemEdit:TypeString"
+
 static void
-setup_combo (GnomeDItemEdit *dee,
-	     int             type,
-	     const char     *extra)
+add_menuitem (GtkWidget *menu, const char *str, const char *label,
+	      const char *select, GtkWidget **selected)
 {
-	GList *types = NULL;
+	GtkWidget *item = gtk_menu_item_new_with_label (label);
+	gtk_widget_show (item);
+	g_object_set_data_full (G_OBJECT (item), TYPE_STRING,
+				g_strdup (str),
+				(GDestroyNotify)g_free);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+
+	if (select != NULL &&
+	    strcmp (str, select) == 0)
+		*selected = item;
+}
+
+static void
+setup_option (GnomeDItemEdit *dee,
+	      int             type,
+	      const char     *select)
+{
+	GtkWidget *menu;
+	GtkWidget *selected = NULL;
+
+	menu = gtk_menu_new ();
 
 	switch (type) {
 	case ONLY_DIRECTORY:
-		types = g_list_prepend (types, "Directory");
+		add_menuitem (menu, "Directory", _("Directory"),
+			      select, &selected);
 		break;
 	default: 
-		types = g_list_prepend (types, "Application");
+		add_menuitem (menu, "Application", _("Application"),
+			      select, &selected);
 
 		if (type != ALL_EXCEPT_DIRECTORY)
-			types = g_list_prepend (types, "Directory");
+			add_menuitem (menu, "Directory", _("Directory"),
+				      select, &selected);
 
-		types = g_list_prepend (types, "Link");
-		types = g_list_prepend (types, "FSDevice");
-		types = g_list_prepend (types, "MimeType");
-		types = g_list_prepend (types, "Service");
-		types = g_list_prepend (types, "ServiceType");
+		add_menuitem (menu, "Link", _("Link"),
+			      select, &selected);
+		add_menuitem (menu, "FSDevice", _("FSDevice"),
+			      select, &selected);
+		add_menuitem (menu, "MimeType", _("MimeType"),
+			      select, &selected);
+		add_menuitem (menu, "Service", _("Service"),
+			      select, &selected);
+		add_menuitem (menu, "ServiceType", _("ServiceType"),
+			      select, &selected);
 		break;
 	}
 
-	if (extra != NULL)
-		types = g_list_prepend (types, (char *) extra);
+	if (select != NULL &&
+	    selected == NULL)
+		add_menuitem (menu, select, _(select), select, &selected);
 
-	g_assert (types != NULL);
+	if (selected != NULL)
+		gtk_menu_item_activate (GTK_MENU_ITEM (selected));
 
-	gtk_combo_set_popdown_strings (
-		GTK_COMBO (dee->_priv->type_combo), types);
-
-	g_list_free (types);
+	gtk_option_menu_set_menu (GTK_OPTION_MENU (dee->_priv->type_option), menu);
 }
 
+static const char *
+get_type_from_option (GnomeDItemEdit *dee)
+{
+	GtkWidget *menu, *active;
+
+	menu = gtk_option_menu_get_menu (GTK_OPTION_MENU (dee->_priv->type_option));
+	if (menu == NULL)
+		return NULL;
+
+	active = gtk_menu_get_active (GTK_MENU (menu));
+	if (active == NULL)
+		return NULL;
+
+	return g_object_get_data (G_OBJECT (active), TYPE_STRING);
+}
+
+
 static void 
 table_attach_entry (GtkTable  *table,
 		    GtkWidget *entry,
@@ -230,16 +279,6 @@ table_attach_label (GtkTable  *table,
 		GTK_FILL, GTK_FILL, 0, 0);
 }
 
-static void 
-table_attach_list(GtkTable * table, GtkWidget * w,
-		  gint l, gint r, gint t, gint b)
-{
-        gtk_table_attach(table, w, l, r, t, b,
-                         GTK_EXPAND | GTK_FILL | GTK_SHRINK,
-                         GTK_EXPAND | GTK_FILL | GTK_SHRINK,
-                         0, 0);
-}
-
 static GtkWidget *
 label_new (const char *text)
 {
@@ -252,31 +291,44 @@ label_new (const char *text)
 
 /* A hack! */
 static void
-type_combo_changed (GnomeDItemEdit *dee)
+type_option_changed (GnomeDItemEdit *dee)
 {
 	const char *type;
-        type = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (dee->_priv->type_combo)->entry));
+	type = get_type_from_option (dee);
 	if (type != NULL &&
 	    strcmp (type, "Link") == 0 /* URL */)
-		gtk_label_set_text (GTK_LABEL (dee->_priv->type_label),
+		gtk_label_set_text (GTK_LABEL (dee->_priv->exec_label),
 				    _("URL:"));
 	else
-		gtk_label_set_text (GTK_LABEL (dee->_priv->type_label),
-				    _("Type:"));
+		gtk_label_set_text (GTK_LABEL (dee->_priv->exec_label),
+				    _("Command:"));
 }
 
-static void
-fill_easy_page (GnomeDItemEdit *dee,
-		GtkWidget      *table)
+static GtkWidget *
+make_easy_page (GnomeDItemEdit *dee)
 {
+	GtkWidget *frame;
+	GtkWidget *table;
 	GtkWidget *label;
 	GtkWidget *entry;
 	GtkWidget *hbox;
 	GtkWidget *align;
-	GtkWidget *combo;
+	GtkWidget *option;
 	GtkWidget *icon_entry;
 	GtkWidget *check_button;
 
+        frame = gtk_frame_new (NULL);
+        gtk_container_set_border_width (
+			GTK_CONTAINER (frame), GNOME_PAD_SMALL);
+
+        table = gtk_table_new (5, 2, FALSE);
+        gtk_container_set_border_width (GTK_CONTAINER (table), GNOME_PAD_SMALL);
+        gtk_table_set_row_spacings (GTK_TABLE (table), GNOME_PAD_SMALL);
+        gtk_table_set_col_spacings (GTK_TABLE (table), GNOME_PAD_SMALL);
+
+        gtk_container_add (GTK_CONTAINER (frame), table);
+
+	/* Name */
 	label = label_new (_("Name:"));
 	table_attach_label (GTK_TABLE (table), label, 0, 1, 0, 1);
 
@@ -294,12 +346,31 @@ fill_easy_page (GnomeDItemEdit *dee,
 
 	set_relation (dee->_priv->name_entry, GTK_LABEL (label));
 
-	label = label_new (_("Comment:"));
+	/* Generic Name */
+	label = label_new (_("Generic name:"));
 	table_attach_label (GTK_TABLE (table), label, 0, 1, 1, 2);
 
 	entry = gtk_entry_new ();
 	table_attach_entry (GTK_TABLE (table), entry, 1, 2, 1, 2);
 
+	g_signal_connect_object (entry, "changed",
+				 G_CALLBACK (gnome_ditem_edit_changed),
+				 dee, G_CONNECT_SWAPPED);		
+
+	g_signal_connect_object (entry, "changed",
+				 G_CALLBACK (gnome_ditem_edit_name_changed),
+				 dee, G_CONNECT_SWAPPED);
+	dee->_priv->generic_name_entry = entry;
+
+	set_relation (dee->_priv->generic_name_entry, GTK_LABEL (label));
+
+	/* Comment */
+	label = label_new (_("Comment:"));
+	table_attach_label (GTK_TABLE (table), label, 0, 1, 2, 3);
+
+	entry = gtk_entry_new ();
+	table_attach_entry (GTK_TABLE (table), entry, 1, 2, 2, 3);
+
 	g_signal_connect_object (entry, "changed",
 				 G_CALLBACK (gnome_ditem_edit_changed),
                                  dee, G_CONNECT_SWAPPED);
@@ -308,11 +379,11 @@ fill_easy_page (GnomeDItemEdit *dee,
 	set_relation (dee->_priv->comment_entry, GTK_LABEL (label));
 
 	label = label_new (_("Command:"));
-	table_attach_label (GTK_TABLE (table), label, 0, 1, 2, 3);
+	table_attach_label (GTK_TABLE (table), label, 0, 1, 3, 4);
 	dee->_priv->exec_label = label;
 
-	entry = gtk_entry_new ();
-	table_attach_entry (GTK_TABLE (table), entry, 1, 2, 2, 3);
+	entry = gnome_file_entry_new ("command", _("Browse"));
+	table_attach_entry (GTK_TABLE (table), entry, 1, 2, 3, 4);
  
 	g_signal_connect_object (entry, "changed",
 				 G_CALLBACK (gnome_ditem_edit_changed),
@@ -322,32 +393,32 @@ fill_easy_page (GnomeDItemEdit *dee,
 	set_relation (dee->_priv->exec_entry, GTK_LABEL (label));
 
 	label = label_new (_("Type:"));
-	table_attach_label (GTK_TABLE (table), label, 0, 1, 3, 4);
+	table_attach_label (GTK_TABLE (table), label, 0, 1, 4, 5);
 	dee->_priv->type_label = label;
 
-	dee->_priv->type_combo = combo = gtk_combo_new ();
-	setup_combo (dee, ALL_TYPES, NULL);
+	dee->_priv->type_option = option = gtk_option_menu_new ();
+	setup_option (dee, ALL_TYPES, NULL);
 
-	gtk_combo_set_value_in_list (GTK_COMBO (combo), FALSE, TRUE);
-	table_attach_entry (GTK_TABLE (table), combo, 1, 2, 3, 4);
+	table_attach_entry (GTK_TABLE (table), option, 1, 2, 4, 5);
 
-	g_signal_connect_object (GTK_COMBO (combo)->entry, "changed",
+	g_signal_connect_object (G_OBJECT (option), "changed",
 				 G_CALLBACK (gnome_ditem_edit_changed),
 				 dee, G_CONNECT_SWAPPED);	
-	g_signal_connect_object (GTK_COMBO (combo)->entry, "changed",
-				 G_CALLBACK (type_combo_changed),
+	g_signal_connect_object (G_OBJECT (option), "changed",
+				 G_CALLBACK (type_option_changed),
 				 dee, G_CONNECT_SWAPPED);
-	dee->_priv->type_combo = combo;
 
-	set_relation (dee->_priv->type_combo, GTK_LABEL (label));
+	set_relation (dee->_priv->type_option, GTK_LABEL (label));
 
 	label = label_new (_("Icon:"));
-	table_attach_label (GTK_TABLE (table), label, 0, 1, 4, 5);
+	table_attach_label (GTK_TABLE (table), label, 0, 1, 5, 6);
 
 	hbox = gtk_hbox_new (FALSE, GNOME_PAD_BIG);
-	gtk_table_attach (GTK_TABLE (table), hbox, 1, 2, 4, 5,
+	gtk_table_attach (GTK_TABLE (table), hbox, 1, 2, 5, 6,
 			  GTK_EXPAND | GTK_FILL, 0, 0, 0); 
 
+	/* FIXME: locale specific icons!!! how the hell do we
+	 * handle that !!! */
         icon_entry = gnome_icon_entry_new (
 			"desktop-icon", _("Browse icons"));
 
@@ -371,6 +442,7 @@ fill_easy_page (GnomeDItemEdit *dee,
         gtk_container_add (GTK_CONTAINER (align), check_button);
         dee->_priv->terminal_button = check_button;
 
+	return frame;
 }
 
 static GtkTreeIter*
@@ -417,18 +489,26 @@ translations_select_row (GtkTreeSelection *selection,
         GtkTreeIter   iter;
         char         *lang;
         char         *name;
+        char         *generic_name;
         char         *comment;
 
 	 if (!gtk_tree_selection_get_selected (selection, &model, &iter))
 		return;
 
 	gtk_tree_model_get (
-		model, &iter, 0, &lang, 1, &name, 2, &comment, -1);
+		model, &iter,
+		0, &lang,
+		1, &name,
+		2, &generic_name,
+		3, &comment,
+		-1);
 
         gtk_entry_set_text(
 		GTK_ENTRY (dee->_priv->transl_lang_entry), lang);
         gtk_entry_set_text (
 		GTK_ENTRY (dee->_priv->transl_name_entry), name);
+        gtk_entry_set_text (
+		GTK_ENTRY (dee->_priv->transl_generic_name_entry), generic_name);
         gtk_entry_set_text(
 		GTK_ENTRY (dee->_priv->transl_comment_entry), comment);
 	
@@ -447,12 +527,14 @@ translations_add (GtkWidget      *button,
 	const GList  *langs;
 	const char   *tmp;
 	const char   *name;
+	const char   *generic_name;
 	const char   *comment;
 	char         *lang;
 	gboolean      ret;
 
 	tmp     = gtk_entry_get_text (GTK_ENTRY (dee->_priv->transl_lang_entry));
 	name    = gtk_entry_get_text (GTK_ENTRY (dee->_priv->transl_name_entry));
+	generic_name = gtk_entry_get_text (GTK_ENTRY (dee->_priv->transl_generic_name_entry));
 	comment = gtk_entry_get_text (GTK_ENTRY (dee->_priv->transl_comment_entry));
 
 	g_assert (tmp != NULL && name != NULL && comment != NULL);
@@ -467,12 +549,21 @@ translations_add (GtkWidget      *button,
 	/*
 	 * If we are editing the current language, change the name and
 	 * comment entries on the easy page as well.
+	 *
+	 * FIXME - this doesn't work. We need something like
+	 * gnome_desktop_item_get_best_locale and compare that with lang.
+	 * Right now for example my locale is en_US, but 
+	 * gnome_i18n_get_language_list returns en_US.iso8859-1 as the first item
+	 * and en_US as the second. gnome-desktop-item handles all this correctly
+	 * and needs a function to return to us the locale it's using as default.
 	 */
-	langs = gnome_i18n_get_language_list ("LC_ALL");
+	langs = gnome_i18n_get_language_list ("LC_MESSAGES");
 	tmp = langs ? langs->data : NULL;
-	if ((tmp && !strcmp (tmp, lang)) || (!tmp && !strcmp (tmp, "C"))) {
+	if ((tmp && !strcmp (tmp, lang)) || (!tmp && !strcmp (lang, "C"))) {
 		gtk_entry_set_text (
 			GTK_ENTRY (dee->_priv->name_entry), name);
+		gtk_entry_set_text (
+			GTK_ENTRY (dee->_priv->generic_name_entry), generic_name);
 		gtk_entry_set_text (
 			GTK_ENTRY (dee->_priv->comment_entry), comment);
 	}
@@ -489,9 +580,12 @@ translations_add (GtkWidget      *button,
 		if (!strcmp (lang, string)) {
 			gtk_list_store_set (
 				GTK_LIST_STORE (model), &iter,
-				1, name, 2, comment, -1);
+				1, name,
+				2, generic_name,
+				3, comment,
+				-1);
 
-			g_signal_emit (dee, ditem_edit_signals [CHANGED], 0);
+			gnome_ditem_edit_changed (dee);
 
 			g_free (string);
 			g_free (lang);
@@ -507,16 +601,22 @@ translations_add (GtkWidget      *button,
 	gtk_list_store_append (GTK_LIST_STORE (model), &iter);
 	gtk_list_store_set (
 		GTK_LIST_STORE (model), &iter,
-		0, lang, 1, name, 2, comment, -1);
+		0, lang,
+		1, name,
+		2, generic_name,
+		3, comment,
+		-1);
 
 	gtk_editable_delete_text (
 		GTK_EDITABLE (dee->_priv->transl_lang_entry), 0, -1);
 	gtk_editable_delete_text (
 		GTK_EDITABLE (dee->_priv->transl_name_entry), 0, -1);
+	gtk_editable_delete_text (
+		GTK_EDITABLE (dee->_priv->transl_generic_name_entry), 0, -1);
 	gtk_editable_delete_text (
 		GTK_EDITABLE (dee->_priv->transl_comment_entry), 0, -1);
 
-	g_signal_emit (dee, ditem_edit_signals [CHANGED], 0);
+	gnome_ditem_edit_changed (dee);
 
 	g_free (lang);
 }
@@ -537,8 +637,8 @@ translations_remove (GtkWidget      *button,
                return;
 
         gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
-
-        g_signal_emit (G_OBJECT(dee), ditem_edit_signals [CHANGED], 0);
+	
+	gnome_ditem_edit_changed (dee);
 }
 
 static GtkWidget *
@@ -550,7 +650,7 @@ setup_translations_list (GnomeDItemEdit *dee)
 	GtkListStore      *model;
 	GtkWidget         *tree;
 
-	model = gtk_list_store_new (3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
+	model = gtk_list_store_new (4, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
 	tree = gtk_tree_view_new_with_model (GTK_TREE_MODEL (model));
 	g_object_unref (model);
 
@@ -567,10 +667,15 @@ setup_translations_list (GnomeDItemEdit *dee)
 	gtk_tree_view_append_column (GTK_TREE_VIEW (tree), column);
 
 	column = gtk_tree_view_column_new_with_attributes (
-					_("Comment"), renderer,
+					_("Generic name"), renderer,
 					"text", 2, NULL);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (tree), column);
 
+	column = gtk_tree_view_column_new_with_attributes (
+					_("Comment"), renderer,
+					"text", 3, NULL);
+	gtk_tree_view_append_column (GTK_TREE_VIEW (tree), column);
+
 	gtk_tree_view_columns_autosize (GTK_TREE_VIEW (tree));
 
 	gtk_tree_view_set_headers_clickable (GTK_TREE_VIEW (tree), FALSE);
@@ -583,21 +688,37 @@ setup_translations_list (GnomeDItemEdit *dee)
 	return tree;
 }
  
-static void
-fill_advanced_page (GnomeDItemEdit *dee,
-		    GtkWidget      *page)
+static GtkWidget *
+make_advanced_page (GnomeDItemEdit *dee)
 {
+	GtkWidget *frame;
+	GtkWidget *vbox;
+	GtkWidget *table;
 	GtkWidget *label;
 	GtkWidget *entry;
 	GtkWidget *button;
 	GtkWidget *box;
 
+        frame = gtk_frame_new (NULL);
+        gtk_container_set_border_width (
+			GTK_CONTAINER (frame), GNOME_PAD_SMALL);
+
+	vbox = gtk_vbox_new (FALSE, GNOME_PAD_SMALL);
+        gtk_container_add (GTK_CONTAINER (frame), vbox);
+
+        table = gtk_table_new (2, 2, FALSE);
+        gtk_container_set_border_width (GTK_CONTAINER (table), GNOME_PAD_SMALL);
+        gtk_table_set_row_spacings (GTK_TABLE (table), GNOME_PAD_SMALL);
+        gtk_table_set_col_spacings (GTK_TABLE (table), GNOME_PAD_SMALL);
+
+        gtk_box_pack_start (GTK_BOX (vbox), table, FALSE, FALSE, 0);
+
 	label = label_new (_("Try this before using:"));
-	table_attach_label (GTK_TABLE (page), label, 0, 1, 0, 1);
+	table_attach_label (GTK_TABLE (table), label, 0, 1, 0, 1);
 	dee->_priv->tryexec_label = label;
 
 	entry = gtk_entry_new ();
-	table_attach_entry (GTK_TABLE (page), entry, 1, 2, 0, 1);
+	table_attach_entry (GTK_TABLE (table), entry, 1, 2, 0, 1);
 	g_signal_connect_swapped (entry, "changed",
 				  G_CALLBACK (gnome_ditem_edit_changed), dee);
 	dee->_priv->tryexec_entry = entry;
@@ -605,11 +726,11 @@ fill_advanced_page (GnomeDItemEdit *dee,
 	set_relation (dee->_priv->tryexec_entry, GTK_LABEL (label));
 
 	label = label_new (_("Documentation:"));
-	table_attach_label (GTK_TABLE (page), label, 0, 1, 1, 2);
+	table_attach_label (GTK_TABLE (table), label, 0, 1, 1, 2);
 
 	entry = gtk_entry_new ();
 	gtk_entry_set_max_length (GTK_ENTRY (entry), 255);        
-	table_attach_entry (GTK_TABLE (page), entry, 1, 2, 1, 2);
+	table_attach_entry (GTK_TABLE (table), entry, 1, 2, 1, 2);
 	g_signal_connect_swapped (entry, "changed",
 				  G_CALLBACK (gnome_ditem_edit_changed), dee);
 	dee->_priv->doc_entry = entry;
@@ -617,10 +738,18 @@ fill_advanced_page (GnomeDItemEdit *dee,
 	set_relation (dee->_priv->doc_entry, GTK_LABEL (label));
 
 	label = gtk_label_new (_("Name/Comment translations:"));
-	table_attach_label (GTK_TABLE (page), label, 0, 2, 2, 3);
+	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
+
+	dee->_priv->translations = setup_translations_list (dee);
+
+	box = gtk_scrolled_window_new (NULL, NULL);
+	gtk_widget_set_size_request (box, 0, 120);
+	gtk_container_add (GTK_CONTAINER (box), dee->_priv->translations);
+	gtk_box_pack_start (GTK_BOX (vbox), box, TRUE, TRUE, 0);
 
 	box = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);
-	table_attach_entry (GTK_TABLE (page), box, 0, 2, 3, 4);
+	gtk_box_pack_start (GTK_BOX (vbox), box, FALSE, FALSE, 0);
 
 	entry = gtk_entry_new ();
 	gtk_box_pack_start (GTK_BOX (box), entry, TRUE, TRUE, 0);
@@ -632,6 +761,7 @@ fill_advanced_page (GnomeDItemEdit *dee,
 
 	entry = gtk_entry_new ();
 	gtk_box_pack_start (GTK_BOX (box), entry, TRUE, TRUE, 0);
+	gtk_widget_set_size_request (entry, 80, -1);
 	dee->_priv->transl_name_entry = entry;
 
 	set_tooltip (dee, GTK_WIDGET(entry), _("Name"));
@@ -639,13 +769,22 @@ fill_advanced_page (GnomeDItemEdit *dee,
 
 	entry = gtk_entry_new ();
 	gtk_box_pack_start (GTK_BOX (box), entry, TRUE, TRUE, 0);
+	gtk_widget_set_size_request (entry, 80, -1);
+	dee->_priv->transl_generic_name_entry = entry;
+
+	set_tooltip (dee, GTK_WIDGET (entry), _("Generic name"));
+	set_relation (dee->_priv->transl_generic_name_entry, GTK_LABEL (label));
+
+	/* FIXME: transl_icon_entry, locale specific icons */
+
+	entry = gtk_entry_new ();
+	gtk_box_pack_start (GTK_BOX (box), entry, TRUE, TRUE, 0);
+	gtk_widget_set_size_request (entry, 80, -1);
 	dee->_priv->transl_comment_entry = entry;
 
 	set_tooltip (dee, GTK_WIDGET(entry), _("Comment"));
 	set_relation (dee->_priv->transl_comment_entry, GTK_LABEL (label));
 
-	box = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);
-	table_attach_entry (GTK_TABLE (page), box, 0, 2, 4, 5);
 	button = gtk_button_new_with_label (_("Add/Set"));
 	gtk_box_pack_start (GTK_BOX (box), button, FALSE, FALSE, 0);
 	g_signal_connect (button, "clicked",
@@ -661,34 +800,9 @@ fill_advanced_page (GnomeDItemEdit *dee,
 	set_tooltip (dee, GTK_WIDGET(button),
 		    _("Remove Name/Comment Translation"));
   
-	dee->_priv->translations = setup_translations_list (dee);
-
-	box = gtk_scrolled_window_new (NULL, NULL);
-	gtk_widget_set_size_request (box, 0, 120);
-	gtk_container_add (GTK_CONTAINER (box), dee->_priv->translations);
-	table_attach_list (GTK_TABLE (page), box, 0, 2, 5, 6);
+	return frame;
 }
 
-static GtkWidget *
-make_page (void)
-{
-        GtkWidget *frame;
-	GtkWidget *page;
-
-        frame = gtk_frame_new (NULL);
-        gtk_container_set_border_width (
-			GTK_CONTAINER (frame), GNOME_PAD_SMALL);
-
-        page = gtk_table_new (5, 2, FALSE);
-        gtk_container_set_border_width (GTK_CONTAINER (page), GNOME_PAD_SMALL);
-        gtk_table_set_row_spacings (GTK_TABLE (page), GNOME_PAD_SMALL);
-        gtk_table_set_col_spacings (GTK_TABLE (page), GNOME_PAD_SMALL);
-
-        gtk_container_add (GTK_CONTAINER (frame), page);
-
-        return frame;
-} 
-
 static void
 gnome_ditem_edit_instance_init (GnomeDItemEdit *dee)
 {
@@ -696,8 +810,7 @@ gnome_ditem_edit_instance_init (GnomeDItemEdit *dee)
 
 	dee->_priv = g_new0 (GnomeDItemEditPrivate, 1);
 
-        page = make_page ();
-        fill_easy_page (dee, GTK_BIN (page)->child);
+        page = make_easy_page (dee);
         gtk_widget_show_all (page);
 
         gtk_notebook_append_page (GTK_NOTEBOOK (dee), page,
@@ -705,8 +818,7 @@ gnome_ditem_edit_instance_init (GnomeDItemEdit *dee)
 
 	dee->_priv->child1 = page;
 
-        page = make_page ();
-        fill_advanced_page (dee, GTK_BIN (page)->child);
+        page = make_advanced_page (dee);
         gtk_widget_show_all (page);
 
         gtk_notebook_append_page (GTK_NOTEBOOK (dee), page,
@@ -784,7 +896,7 @@ gnome_ditem_set_directory_sensitive (GnomeDItemEdit *dee,
 	gtk_widget_set_sensitive (dee->_priv->exec_label, ! is_directory);
 	gtk_widget_set_sensitive (dee->_priv->exec_entry, ! is_directory);
 	gtk_widget_set_sensitive (dee->_priv->type_label, ! is_directory);
-	gtk_widget_set_sensitive (dee->_priv->type_combo, ! is_directory);
+	gtk_widget_set_sensitive (dee->_priv->type_option, ! is_directory);
 	gtk_widget_set_sensitive (dee->_priv->terminal_button, ! is_directory);
 	gtk_widget_set_sensitive (dee->_priv->tryexec_label, ! is_directory);
 	gtk_widget_set_sensitive (dee->_priv->tryexec_entry, ! is_directory);
@@ -794,13 +906,18 @@ gnome_ditem_set_directory_sensitive (GnomeDItemEdit *dee,
 static void
 gnome_ditem_edit_sync_display (GnomeDItemEdit *dee)
 {
-        GList *i18n_list, *li;
-	GnomeDesktopItemType type;
-        const gchar* cs;
-        char* tmpstr;
-	GnomeDesktopItem *ditem;
-        GtkTreeModel *model; 
-        GtkTreeIter iter;
+	GnomeDesktopItemType  type;
+	GnomeDesktopItem    *ditem;
+        GtkTreeModel        *model; 
+        GtkTreeIter          iter;
+        GList                *i18n_list;
+        GList                *li;
+        const char           *cs;
+        const char           *name;
+        const char           *generic_name;
+        const char           *comment;
+        char                 *tmpstr;
+
         g_return_if_fail (dee != NULL);
         g_return_if_fail (GNOME_IS_DITEM_EDIT (dee));
 
@@ -812,29 +929,31 @@ gnome_ditem_edit_sync_display (GnomeDItemEdit *dee)
 	}
 
 	type = gnome_desktop_item_get_entry_type (ditem);
+        cs = gnome_desktop_item_get_string (ditem,
+					    GNOME_DESKTOP_ITEM_TYPE);
 	if (type == GNOME_DESKTOP_ITEM_TYPE_DIRECTORY ||
 	    dee->_priv->directory_only) {
 		gnome_ditem_set_directory_sensitive (dee, TRUE);
-		setup_combo (dee, ONLY_DIRECTORY, NULL);
+		setup_option (dee, ONLY_DIRECTORY, cs);
 	} else {
-		const char *extra = NULL;
 		gnome_ditem_set_directory_sensitive (dee, FALSE);
-		if (type == GNOME_DESKTOP_ITEM_TYPE_OTHER) {
-			extra = gnome_desktop_item_get_string
-				(ditem, GNOME_DESKTOP_ITEM_TYPE);
-		}
-		setup_combo (dee, ALL_EXCEPT_DIRECTORY, extra);
+		setup_option (dee, ALL_EXCEPT_DIRECTORY, cs);
 	}
 
-        cs = gnome_desktop_item_get_localestring
-		(ditem, GNOME_DESKTOP_ITEM_NAME);
-        gtk_entry_set_text(GTK_ENTRY(dee->_priv->name_entry), 
-                           cs ? cs : "");
+        name = gnome_desktop_item_get_localestring (
+			ditem, GNOME_DESKTOP_ITEM_NAME);
+        gtk_entry_set_text (GTK_ENTRY (dee->_priv->name_entry), 
+			    name ? name : "");
 
-        cs = gnome_desktop_item_get_localestring
-		(ditem, GNOME_DESKTOP_ITEM_COMMENT);
-        gtk_entry_set_text(GTK_ENTRY(dee->_priv->comment_entry),
-                           cs ? cs : "");
+        generic_name = gnome_desktop_item_get_localestring (
+				ditem, GNOME_DESKTOP_ITEM_GENERIC_NAME);
+        gtk_entry_set_text (GTK_ENTRY (dee->_priv->generic_name_entry), 
+			    generic_name ? generic_name : "");
+
+        comment = gnome_desktop_item_get_localestring (
+				ditem, GNOME_DESKTOP_ITEM_COMMENT);
+        gtk_entry_set_text (GTK_ENTRY (dee->_priv->comment_entry),
+			    comment ? comment : "");
 
 	if (type == GNOME_DESKTOP_ITEM_TYPE_LINK) {
 		cs = gnome_desktop_item_get_string (ditem,
@@ -843,24 +962,21 @@ gnome_ditem_edit_sync_display (GnomeDItemEdit *dee)
 		cs = gnome_desktop_item_get_string (ditem,
 						    GNOME_DESKTOP_ITEM_EXEC);
 	}
-        gtk_entry_set_text(GTK_ENTRY(dee->_priv->exec_entry), cs ? cs : "");
+        gnome_file_entry_set_filename
+		(GNOME_FILE_ENTRY (dee->_priv->exec_entry), cs ? cs : "");
 
         cs = gnome_desktop_item_get_string (ditem,
 					    GNOME_DESKTOP_ITEM_TRY_EXEC);
         gtk_entry_set_text(GTK_ENTRY(dee->_priv->tryexec_entry), 
                            cs ? cs : "");
 
-        tmpstr = gnome_desktop_item_get_icon (ditem);
+        tmpstr = gnome_desktop_item_get_icon (ditem, NULL);
 	gnome_icon_entry_set_filename (GNOME_ICON_ENTRY (dee->_priv->icon_entry), tmpstr);
 	g_free (tmpstr);
 
         cs = gnome_desktop_item_get_string (ditem, "DocPath"); /* FIXME check name */
         gtk_entry_set_text (GTK_ENTRY (dee->_priv->doc_entry), cs ? cs : "");
 
-        cs = gnome_desktop_item_get_string (ditem, GNOME_DESKTOP_ITEM_TYPE);
-        gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (dee->_priv->type_combo)->entry),
-			    cs ? cs : "");
-
         gtk_toggle_button_set_active
 		(GTK_TOGGLE_BUTTON (dee->_priv->terminal_button),
 		 gnome_desktop_item_get_boolean (ditem,
@@ -870,26 +986,36 @@ gnome_ditem_edit_sync_display (GnomeDItemEdit *dee)
         model = gtk_tree_view_get_model (GTK_TREE_VIEW (dee->_priv->translations));
         gtk_list_store_clear (GTK_LIST_STORE (model));
         i18n_list = gnome_desktop_item_get_languages (ditem, NULL);
-        for (li = i18n_list; li != NULL; li = li->next) {
-                const char *text[3];
-                const gchar* lang = li->data;
-                cs = lang;
-                text[0] = cs ? cs : "";
-                cs = gnome_desktop_item_get_localestring_lang
+	for (li = i18n_list; li != NULL; li = li->next) {
+                const char *name, *comment, *generic_name;
+                const char *lang = li->data;
+
+                name = gnome_desktop_item_get_localestring_lang
 			(ditem, GNOME_DESKTOP_ITEM_NAME, lang);
-                text[1] = cs ? cs : "";
-                cs = gnome_desktop_item_get_localestring_lang
+                generic_name = gnome_desktop_item_get_localestring_lang
+			(ditem, GNOME_DESKTOP_ITEM_GENERIC_NAME, lang);
+                comment = gnome_desktop_item_get_localestring_lang
 			(ditem, GNOME_DESKTOP_ITEM_COMMENT, lang);
-                text[2] = cs ? cs : "";
-                gtk_list_store_append (GTK_LIST_STORE(model), &iter);
-                gtk_list_store_set (GTK_LIST_STORE(model),&iter, 0, 
-                                    text[0],1,text[1],2,text[2],-1);
+                
+		/* only include a language in the list if it 
+		 * has a useful translation
+		 */
+		if (name || generic_name || comment) {
+			gtk_list_store_append (GTK_LIST_STORE(model), &iter);
+			gtk_list_store_set (GTK_LIST_STORE (model), &iter,
+					    0, lang ? lang : "",
+					    1, name ? name : "",
+					    2, generic_name ? generic_name : "",
+					    3, comment ? comment : "",
+					    -1);
+		}
         }
         g_list_free (i18n_list);
 
 	/* clear the entries for add/remove */
         gtk_entry_set_text (GTK_ENTRY (dee->_priv->transl_lang_entry), "");
         gtk_entry_set_text (GTK_ENTRY (dee->_priv->transl_name_entry), "");
+        gtk_entry_set_text (GTK_ENTRY (dee->_priv->transl_generic_name_entry), "");
         gtk_entry_set_text (GTK_ENTRY (dee->_priv->transl_comment_entry), "");
 
 	/* ui can't be dirty, I mean, damn we just synced it from the ditem */
@@ -917,6 +1043,7 @@ gnome_ditem_edit_sync_ditem (GnomeDItemEdit *dee)
 	GnomeDesktopItem *ditem;
 	GtkTreeModel     *model;
 	GtkTreeIter       iter;
+	GtkWidget        *entry;
 	const char       *type;
 	const char       *uri;
 	const char       *attr;
@@ -931,12 +1058,12 @@ gnome_ditem_edit_sync_ditem (GnomeDItemEdit *dee)
 
 	ditem = dee->_priv->ditem;
 
-	uri = gtk_entry_get_text (GTK_ENTRY (dee->_priv->exec_entry));
+	entry = gnome_file_entry_gtk_entry
+		(GNOME_FILE_ENTRY (dee->_priv->exec_entry));
+	uri = gtk_entry_get_text (GTK_ENTRY (entry));
 
-	type = gtk_entry_get_text (
-			GTK_ENTRY (GTK_COMBO (dee->_priv->type_combo)->entry));
-	gnome_desktop_item_set_string (
-			ditem, GNOME_DESKTOP_ITEM_TYPE, type);
+	type = get_type_from_option (dee);
+	gnome_desktop_item_set_string (ditem, GNOME_DESKTOP_ITEM_TYPE, type);
 
 	/* hack really */
 	if (type && !strcmp (type, "Link"))
@@ -966,6 +1093,8 @@ gnome_ditem_edit_sync_ditem (GnomeDItemEdit *dee)
 
 	gnome_desktop_item_clear_localestring (
 			ditem, GNOME_DESKTOP_ITEM_NAME);
+	gnome_desktop_item_clear_localestring (
+			ditem, GNOME_DESKTOP_ITEM_GENERIC_NAME);
 	gnome_desktop_item_clear_localestring (
 			ditem, GNOME_DESKTOP_ITEM_COMMENT);
 
@@ -977,9 +1106,10 @@ gnome_ditem_edit_sync_ditem (GnomeDItemEdit *dee)
 		char *lang;
 		char *name;
 		char *comment;
+		char *generic_name;
 
 		gtk_tree_model_get (
-			model, &iter, 0, &lang, 1, &name, 2, &comment, -1);
+			model, &iter, 0, &lang, 1, &name, 2, &generic_name, 3, &comment, -1);
 
 		if (!name && !comment) {
 			g_free (lang);
@@ -992,10 +1122,13 @@ gnome_ditem_edit_sync_ditem (GnomeDItemEdit *dee)
 
 		gnome_desktop_item_set_localestring_lang (
 			ditem, GNOME_DESKTOP_ITEM_NAME, lang, name);
+		gnome_desktop_item_set_localestring_lang (
+			ditem, GNOME_DESKTOP_ITEM_GENERIC_NAME, lang, generic_name);
 		gnome_desktop_item_set_localestring_lang (
 			ditem, GNOME_DESKTOP_ITEM_COMMENT, lang, comment);
 		
 		g_free (name);
+		g_free (generic_name);
 		g_free (comment);
 		g_free (lang);
 
@@ -1006,6 +1139,10 @@ gnome_ditem_edit_sync_ditem (GnomeDItemEdit *dee)
 			ditem, GNOME_DESKTOP_ITEM_NAME,
 			gtk_entry_get_text (GTK_ENTRY(dee->_priv->name_entry)));
 
+	gnome_desktop_item_set_localestring (
+			ditem, GNOME_DESKTOP_ITEM_GENERIC_NAME,
+			gtk_entry_get_text (GTK_ENTRY(dee->_priv->generic_name_entry)));
+
 	gnome_desktop_item_set_localestring (
 			ditem, GNOME_DESKTOP_ITEM_COMMENT,
 			gtk_entry_get_text (GTK_ENTRY(dee->_priv->comment_entry)));
@@ -1129,6 +1266,7 @@ gnome_ditem_edit_clear (GnomeDItemEdit *dee)
 	dee->_priv->ui_dirty = TRUE;
 
         gtk_entry_set_text(GTK_ENTRY(dee->_priv->name_entry), "");
+        gtk_entry_set_text(GTK_ENTRY(dee->_priv->generic_name_entry), "");
         gtk_entry_set_text(GTK_ENTRY(dee->_priv->comment_entry),"");
         gtk_entry_set_text(GTK_ENTRY(dee->_priv->exec_entry), "");  
         gtk_entry_set_text(GTK_ENTRY(dee->_priv->tryexec_entry), "");
@@ -1136,6 +1274,7 @@ gnome_ditem_edit_clear (GnomeDItemEdit *dee)
         gnome_icon_entry_set_filename (GNOME_ICON_ENTRY (dee->_priv->icon_entry), "");
         gtk_entry_set_text(GTK_ENTRY(dee->_priv->transl_lang_entry), "");
         gtk_entry_set_text(GTK_ENTRY(dee->_priv->transl_name_entry), "");
+        gtk_entry_set_text(GTK_ENTRY(dee->_priv->transl_generic_name_entry), "");
         gtk_entry_set_text(GTK_ENTRY(dee->_priv->transl_comment_entry), "");
         model = gtk_tree_view_get_model (GTK_TREE_VIEW (dee->_priv->translations));
         gtk_list_store_clear (GTK_LIST_STORE (model));
@@ -1143,7 +1282,7 @@ gnome_ditem_edit_clear (GnomeDItemEdit *dee)
 	gnome_ditem_set_directory_sensitive (dee, FALSE);
 
 	/* put all our possibilities here */
-	setup_combo (dee, ALL_TYPES, NULL);
+	setup_option (dee, ALL_TYPES, NULL /* select */);
 }
 
 static void
@@ -1230,11 +1369,26 @@ void
 gnome_ditem_edit_set_entry_type (GnomeDItemEdit *dee,
 				 const char     *type)
 {
+	const char *prev;
+
         g_return_if_fail (dee != NULL);
         g_return_if_fail (GNOME_IS_DITEM_EDIT (dee));
+        g_return_if_fail (type != NULL);
 
-        gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (dee->_priv->type_combo)->entry),
-			    type ? type : "");
+	prev = get_type_from_option (dee);
+	if (prev != NULL &&
+	    strcmp (prev, type) == 0)
+		return;
+
+	if (dee->_priv->directory_only) {
+		gnome_ditem_set_directory_sensitive (dee, TRUE);
+		setup_option (dee, ONLY_DIRECTORY, type);
+		g_signal_emit (dee, ditem_edit_signals [CHANGED], 0);
+	} else {
+		gnome_ditem_set_directory_sensitive (dee, FALSE);
+		setup_option (dee, ALL_EXCEPT_DIRECTORY, type);
+		g_signal_emit (dee, ditem_edit_signals [CHANGED], 0);
+	}
 }
 
 /* force directory only */
@@ -1249,27 +1403,25 @@ gnome_ditem_edit_set_directory_only (GnomeDItemEdit *dee,
 		dee->_priv->directory_only = directory_only;
 
 		if (directory_only) {
-			gnome_ditem_set_directory_sensitive (dee, TRUE);
 			gnome_ditem_edit_set_entry_type (dee, "Directory");
-			setup_combo (dee, ONLY_DIRECTORY, NULL);
 		} else if (dee->_priv->ditem != NULL) {
+			const char *cs = NULL;
 			GnomeDesktopItemType type;
-			type = gnome_desktop_item_get_entry_type (dee->_priv->ditem);
+			type = gnome_desktop_item_get_entry_type
+				(dee->_priv->ditem);
+			cs = gnome_desktop_item_get_string
+				(dee->_priv->ditem, GNOME_DESKTOP_ITEM_TYPE);
 			if (type == GNOME_DESKTOP_ITEM_TYPE_DIRECTORY) {
 				gnome_ditem_set_directory_sensitive (dee, TRUE);
-				setup_combo (dee, ONLY_DIRECTORY, NULL);
+				setup_option (dee, ONLY_DIRECTORY, cs);
 			} else {
-				const char *extra = NULL;
 				gnome_ditem_set_directory_sensitive (dee, FALSE);
-				if (type == GNOME_DESKTOP_ITEM_TYPE_OTHER) {
-					extra = gnome_desktop_item_get_string
-						(dee->_priv->ditem, GNOME_DESKTOP_ITEM_TYPE);
-				}
-				setup_combo (dee, ALL_EXCEPT_DIRECTORY, extra);
+				setup_option (dee, ALL_EXCEPT_DIRECTORY, cs);
 			}
 		} else {
+			const char *type = get_type_from_option (dee);
 			gnome_ditem_set_directory_sensitive (dee, FALSE);
-			setup_combo (dee, ALL_TYPES, NULL /* extra */);
+			setup_option (dee, ALL_TYPES, type);
 		}
 	}
 }
